# Key Fingerprints

!!! hint

    The key fingerprints of login and export nodes can occasionally change. This page holds
    up-to-date fingerprints.

Each cluster can be accessed via so-called **login nodes** using specific hostnames. All login nodes
of a cluster share common SSH key fingerprints that are unique. When connecting to one of our HPC
systems via the corresponding login nodes, please make sure that the provided fingerprint matches
one of the table. This is the only way to ensure you are connecting to the correct server from the
very beginning. If the **fingerprint differs**, please contact the
[HPC support team](../support/support.md).

In case an additional login node of the same cluster is used, the key needs to be checked and
approved again.

??? example "Connecting with SSH to Barnard"

    ```console
    marie@local$ ssh login1.barnard.hpc.tu-dresden.de
    The authenticity of host 'login1.barnard.hpc.tu-dresden.de (172.24.95.28)' can't be established.
    ECDSA key fingerprint is SHA256:8Coljw7yoVH6HA8u+K3makRK9HfOSfe+BG8W/CUEPp0.
    Are you sure you want to continue connecting (yes/no)?
    ```

    In this case, the fingerprint matches the one given in the table. Thus, you can proceed by
    typing 'yes'.

### Barnard

The cluster [Barnard](../jobs_and_resources/barnard.md) can be accessed via the four login
nodes `login[1-4].barnard.hpc.tu-dresden.de`. (Please choose one concrete login node when
connecting, see example below.)

| Key Type | Fingerprint                                         |
|:---------|:----------------------------------------------------|
| RSA      | SHA256:lVQOvnci07jkxmFnX58pQf3cD7lz1mf4K4b9jZrAlVU  |
| RSA      | MD5:5b:39:ae:03:3a:60:15:21:4b:e8:ba:72:52:b8:a1:ad |
| ECDSA    | SHA256:Xan2MYazewT0V5agNazaQfWzLKBD3P48zRwR6reoXhI  |
| ECDSA    | MD5:02:fd:ab:c8:39:f9:94:cc:3f:e0:7e:78:5f:76:b8:4c |
| ED25519  | SHA256:Gn4n5IX9eEvkpOGrtZzs9T9yAfJUB200bgRchchiKAQ  |
| ED25519  | MD5:e8:10:96:67:e8:4c:fd:87:f0:c6:4e:e8:1f:53:a9:be |
{: summary="List of valid fingerprints for Barnard login[1-4] nodes"}

### Romeo

The cluster [Romeo](../jobs_and_resources/romeo.md) can be accessed via the two
login nodes `login[1-2].romeo.hpc.tu-dresden.de`. (Please choose one concrete login node when
connecting, see example below.)

| Key Type | Fingerprint                                         |
|:---------|:----------------------------------------------------|
| RSA      |  |
| RSA      |  |
| ECDSA    |  |
| ECDSA    |  |
| ED25519  |  |
| ED25519  |  |
{: summary="List of valid fingerprints for Romeo login[1-2] node"}

### Alpha Centauri

The cluster [Alpha Centauri](../jobs_and_resources/alpha_centauri.md) can be accessed via the two
login nodes `login[1-2].alpha.hpc.tu-dresden.de`. (Please choose one concrete login node when
connecting, see example below.)

| Key Type | Fingerprint                                         |
|:---------|:----------------------------------------------------|
| RSA      |   |
| RSA      |   |
| ECDSA    |   |
| ECDSA    |   |
| ED25519  |   |
| ED25519  |   |
{: summary="List of valid fingerprints for Alpha Centauri login[1-2] node"}

### Julia

The cluster [Julia](../jobs_and_resources/julia.md) can be accessed via `julia.hpc.tu-dresden.de`.
(Note, there is no separate login node.)

| Key Type | Fingerprint                                         |
|:---------|:----------------------------------------------------|
| RSA      |  |
| RSA      |  |
| ECDSA    |  |
| ECDSA    |  |
| ED25519  |  |
| ED25519  |  |
{: summary="List of valid fingerprints for Julia login node"}

### IBM Power9

The cluster [Power9](../jobs_and_resources/power9.md) can be accessed via the four Taurus login
nodes `tauruslogin[3-6].hrsk.tu-dresden.de` or shortly `taurus.hrsk.tu-dresden.de`.
All of these login nodes share common keys. When connecting, please make sure that the fingerprint
shown matches one of the table.

| Key Type | Fingerprint                                         |
|:---------|:----------------------------------------------------|
| RSA      | SHA256:/M1lW1KTOlxj8UFZJS4tLi+8TyndcDqrZfLGX7KAU8s  |
| RSA      | MD5:b8:e1:21:ed:38:1a:ba:1a:5b:2b:bc:35:31:62:21:49 |
| ECDSA    | SHA256:PeCpW/gAFLvHDzTP2Rb93NxD+rpUsyQY8WebjQC7kz0  |
| ECDSA    | MD5:47:7e:24:46:ab:30:59:2c:1f:e8:fd:37:2a:5d:ee:25 |
| ED25519  | SHA256:nNxjtCny1kB0N0epHaOPeY1YFd0ri2Dvt2CK7rOGlXg  |
| ED25519  | MD5:7c:0c:2b:8b:83:21:b2:08:19:93:6d:03:80:76:8a:7b |
{: summary="List of valid fingerprints for login nodes"}

## Export Nodes

All of these export nodes share common keys. When transfering files, please make sure that the
fingerprint shown matches one of the table.

The following hostnames can be used to transfer files to/from ZIH systems:

- `taurusexport.hpc.tu-dresden.de`
- `taurusexport3.hrsk.tu-dresden.de`
- `taurusexport4.hrsk.tu-dresden.de`

| Key Type | Fingerprint                                         |
|:---------|:----------------------------------------------------|
| RSA      | SHA256:Qjg79R+5x8jlyHhLBZYht599vRk+SujnG1yT1l2dYUM  |
| RSA      | MD5:1e:4c:2d:81:ee:58:1b:d1:3c:0a:18:c4:f7:0b:23:20 |
| ECDSA    | SHA256:qXTZnZMvdqTs3LziA12T1wkhNcFqTHe59fbbU67Qw3g  |
| ECDSA    | MD5:96:62:c6:80:a8:1f:34:64:86:f3:cf:c5:9b:cd:af:da |
| ED25519  | SHA256:jxWiddvDe0E6kpH55PHKF0AaBg/dQLefQaQZ2P4mb3o  |
| ED25519  | MD5:fe:0a:d2:46:10:4a:08:40:fd:e1:99:b7:f2:06:4f:bc |
{: summary="List of valid fingerprints for export nodes"}
